#! python

# Run this file as follows to install the package:
#
#   python setup.py install
#

# Author: Ernesto Posse

import os
import os.path
import sys
import shutil
import stat
from distutils.core import setup
from distutils.cmd import Command
from distutils.command import install
from distutils.version import StrictVersion
import pkg_data

class uninstall (Command):
    
    description = "uninstall package from all installed directories"
    user_options = []
    
    def initialize_options(self):
        pass
    
    def finalize_options(self):
        pass
    
    def run(self):
        do_uninstall()


def preinstall():
    print "* Executing pre-install script"
    try:
        check_dependencies()
        generate_executable()
    except Exception, e:
        print "* (Pre-install error) "+str(e)
    print "* Pre-install script finished"

def check_dependencies():
    print "* Checking for dependencies..."
    failures = 0
    for pkg in pkg_data.dependencies:
        success = check_package(pkg)
        if not success:
            failures += 1
    if failures == 0:
        print "* All required packages are present."
    else:
        print "* There are " + str(failures) + " required packages missing."

def check_package(pkg):
    pkg_name, required_version = pkg
    print "* Checking for package '" + pkg_name + "' >= " +str(required_version)+"..."
    try:
        exec "import " + pkg_name
        module = eval(pkg_name)
        present_version = str(module.version)
        print "* Package '" + pkg_name + " " + present_version + "' was found"
        return check_version(pkg_name, required_version, present_version)
    except ImportError, e:
        print "* Dependency warning: package '" + pkg_name + "' was not found. The installation may not work properly."
    except AttributeError, e:
        print "* Dependency warning: package '" + pkg_name + "' has no known version information. The installation may not work properly."
        return False

def check_version(pkg_name, required_version, present_version):
    print "* Checking package '" + pkg_name + "' version"
    good_enough = StrictVersion(required_version) < StrictVersion(present_version)
    if good_enough:
        print "* Existing package version is good enough" 
    else:
        print "* (Warning) existing package version is older than required version"
    return good_enough

def generate_executable():
    print "* Generating executable script"
    script_text = ""
    script_text += ("#! " + sys.executable).strip() + '\n'
    script_text += "original_script = '" + pkg_data.original_script + "'\n"
    script_text += "import os.path\n"
    script_text += "bin_dir = os.path.abspath(os.path.dirname(__file__))\n"
    script_text += "fullname = os.path.join(bin_dir, original_script)\n"
    script_text += "production = True\n"
    script_text += "execfile(fullname)\n"
    script_file = file(pkg_data.executable_script, 'w')
    script_file.write(script_text)
    script_file.close()
    os.chmod(pkg_data.executable_script, stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)
    print "* Executable script generated and saved"


def postinstall(dist):
    command = dist.get_command_obj('install')
    print "install_base    = '" + command.install_base + "'"
    print "install_purelib = '" + command.install_purelib + "'"
    print "install_platlib = '" + command.install_platlib + "'"
    print "install_lib     = '" + command.install_lib + "'"
    print "install_scripts = '" + command.install_scripts + "'"
    print "install_data    = '" + command.install_data + "'"
    save_install_info(command)
    sys.path += [command.install_lib, command.install_purelib, command.install_platlib]
    if hasattr(pkg_data, 'postinstall_script') \
    and type(pkg_data.postinstall_script) is str \
    and pkg_data.postinstall_script != '':
        execfile(pkg_data.postinstall_script)

def save_install_info(command):
    install_dirs = \
        {
            'base':     command.install_base ,
            'purelib':  command.install_purelib,
            'platlib':  command.install_platlib,
            'lib':      command.install_lib,
            'scripts':  command.install_scripts,
            'data':     command.install_data
        }
    platform = sys.platform.lower()
    if platform.startswith('win'):
        for dir_type in install_dirs:
            dir = install_dirs[dir_type]
            install_dirs[dir_type] = dir.replace('\\','\\\\')
    text = ""
    text += "install_base    = '" + install_dirs['base'] + "'\n"
    text += "install_purelib = '" + install_dirs['purelib'] + "'\n"
    text += "install_platlib = '" + install_dirs['platlib'] + "'\n"
    text += "install_lib     = '" + install_dirs['lib'] + "'\n"
    text += "install_scripts = '" + install_dirs['scripts'] + "'\n"
    text += "install_data    = '" + install_dirs['data'] + "'\n"
    inst_info_file = file('install_data.py','w')
    inst_info_file.write(text)
    inst_info_file.close()

def do_uninstall():
    print "* Uninstalling " + pkg_data.name + ""
    installed_files = file('installed_files.lst','r')
    lines = installed_files.readlines()
    installed_files.close()
    files = [line.strip() for line in lines]
    for f in files:
        try:
            os.remove(f)
        except OSError, e:
            print "* I couldn't remove the file '" + f + "'"
            print "  (OS error) " + str(e)
    uninstall_dirs()
    archive_location = uninstall_distro()
    print "* An archived distribution was left in '" + archive_location + "'"
    print "  You may remove it or reinstall " + pkg_data.name + " from it."
    print "* " + pkg_data.name + " uninstalled"


def uninstall_dirs():
    import install_data
    lib_dirs = \
        [
            install_data.install_lib, 
            install_data.install_purelib, 
            install_data.install_platlib
        ]
    scripts_dir = install_data.install_scripts
    data_dir    = install_data.install_data
    for pkg in pkg_data.packages:
        uninstall_package(pkg, lib_dirs)
    uninstall_data(data_dir)

def uninstall_package(pkg_name, lib_dirs):
    print "* Uninstalling " + pkg_name + " package" 
    #~ try:
        #~ exec "import " + pkg_name
        #~ module = eval(pkg_name)
        #~ pkg_lib_dir        = os.path.abspath(module.__path__[0])
        #~ pkg_lib_parent_dir = os.path.abspath(os.path.dirname(pkg_lib_dir))
        #~ print "* Package '" + pkg_name + "' found at '" + pkg_lib_dir + "'"
    #~ except ImportError, e:
        #~ print "* Package '" + pkg_name + "' not found in the Python path."
        #~ pass
    for lib_dir in lib_dirs:
        lib_dir = os.path.abspath(lib_dir)
        pkg_dir = pkg_name.replace('.', os.path.sep)
        pkg_dir = os.path.join(lib_dir, pkg_dir)
        #~ print "* Trying to remove '" + pkg_dir + "'"
        try:
            shutil.rmtree(pkg_dir)
            print "* Directory '" + pkg_dir + "' removed."
        except OSError, e:
            print "* I couldn't remove '" + pkg_dir + "'. It may have already been erased."
    print "* " + pkg_name + " package uninstalled" 

def uninstall_scripts(scripts_dir):
    print "* Uninstalling " + pkg_data.name + " scripts"
    platform = sys.platform.lower()
    if platform.startswith('linux'):
        bin_dir = 'bin'
    elif platform.startswith('win'):
        bin_dir = 'Scripts'
    full_bin_dir_1 = os.path.abspath(os.path.join(sys.prefix, bin_dir))
    full_bin_dir_2 = os.path.abspath(os.path.join(sys.exec_prefix, bin_dir))
    bin_dirs = [scripts_dir, full_bin_dir_1, full_bin_dir_2]
    for script in pkg_data.scripts:
        script_name = os.path.basename(script)
        root, ext = os.path.splitext(script_name)
        bytecode = root + '.pyc'
        for full_bin_dir in bin_dirs:
            full_path_src = os.path.join(full_bin_dir, script_name)
            full_path_bytecode = os.path.join(full_bin_dir, bytecode)
            if os.path.exists(full_path_src):
                os.remove(full_path_src)
                print "* Removed '" + full_path_src + "'"
            if os.path.exists(full_path_bytecode):
                os.remove(full_path_bytecode)
                print "* Removed '" + full_path_bytecode + "'"
    print "* " + pkg_data.name + " scripts uninstalled"

def uninstall_data(data_dir):
    print "* Uninstalling data file"
    for dir, files in pkg_data.data_files:
        full_dir = os.path.join(data_dir, dir)
        try:
            shutil.rmtree(full_dir)
        except OSError, e:
            print "* I couldn't remove the data directory '" + full_dir + "'"
            print "  (OS error) " + str(e)
    print "* Data files uninstalled"


def uninstall_distro():
    print "* Uninstalling " + pkg_data.name + " distribution"
    distro_dir        = os.path.abspath(os.path.dirname(__file__))
    distro_parent_dir = os.path.abspath(os.path.dirname(distro_dir))
    print "* Recreating distro"
    sys.argv[1] = 'sdist'
    do_setup()
    print "* Removing old distro"
    os.chdir(distro_parent_dir)
    for file_or_dir in os.listdir(distro_dir):
        if file_or_dir != 'dist':
            to_remove = os.path.join(distro_dir, file_or_dir)
            if os.path.isfile(to_remove):
                os.remove(to_remove)
            else:
                shutil.rmtree(to_remove, ignore_errors=False)
    archive_location = os.path.join(distro_dir, 'dist')
    print "* " + pkg_data.name + " distribution uninstalled"
    return archive_location

def do_setup():
    distutils_keys = \
        [
            'name',
            'version',
            'author',
            'author_email',
            'url',
            'package_dir',
            'packages',
            'package_data',
            'py_modules',
            'scripts',
            'data_files',
            'description'
        ]
    distutils_pkg_data = dict((key, pkg_data.__dict__[key]) \
                                for key in distutils_keys \
                                if hasattr(pkg_data, key))
    distutils_pkg_data['cmdclass'] = {'uninstall':uninstall}
    return setup(**distutils_pkg_data)


# main

if sys.argv[1] in ('install', 'build'):
    preinstall()
dist = do_setup()
if sys.argv[1] == 'install':
    postinstall(dist)
    print "* Installation finished"

